@@ -1,6 +1,5 @@
 #############################################################################
 ## package 'secr'
-## methods.R
 ## Methods for classes traps, capthist and mask
 ## Last changed
 ## 2019-10-25 masksize()
@@ -13,6 +12,7 @@
 ## 2022-11-15 read.mask separate file
 ## 2023-04-14 trim.secrlist
 ## 2023-05-30 shift.mask updates boundingbox; explicitly call secr::shift 
+## 2023-08-19 as.popn()  ppp > popn
 ###############################################################################
 
 # Generic methods for extracting attributes etc
@@ -2620,6 +2620,25 @@ as.mask <- function (x) {
 }
 ############################################################################################
 
+as.popn <- function (x) {
+    if (!inherits(x, "ppp")) {        
+        stop ("as.popn is defined only for ppp objects")
+    }       
+    if (requireNamespace("spatstat.geom", quietly = TRUE)) {
+        pop <- spatstat.geom::coords(x)
+        class(pop) <- c('popn', 'data.frame')
+        xl <- x$window$xrange
+        yl <- x$window$yrange
+        attr(pop, 'boundingbox') <- expand.grid (x=xl, y=yl)[c(1,3,4,2),]
+        attr(pop, "Lambda") <- attr(x, "Lambda")   # if present
+        pop
+    }
+    else {
+        stop ("as.popn requires package spatstat")
+    }
+}
+############################################################################################
+
 # derived.default <- function (object, ...)    {
 #     if (!inherits(object, 'secr'))
 #         stop ("no derived method for this class")
